﻿//-----------------------------------------------------------------------------
// Game1.cs
//
// Kyoung Shin Park @ Dankook University (4/4/2011)
//-----------------------------------------------------------------------------
using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;
using Microsoft.Xna.Framework.Net;
using Microsoft.Xna.Framework.Storage;

namespace XNABoxTransformation
{
    /// <summary>
    /// This is the main type for your game
    /// </summary>
    public class Game1 : Microsoft.Xna.Framework.Game
    {
        GraphicsDeviceManager graphics;
        SpriteBatch spriteBatch;

        // effect
        BasicEffect effect;
        Matrix projection;
        Matrix view;
        Matrix world;

        // input
        GamePadState currentGamePadState;
        GamePadState prevGamePadState;
        KeyboardState currentKeyboardState;
        KeyboardState prevKeyboardState;
        bool animationMode = true; 
        bool fillMode = true;
        float angle = 0.0f;
        float zoom = 1.0f;

        // vertex declaration for user defined primitives
        VertexPositionColor[] vertices;
        short[] line_Indices;
        short[] polygon_Indices;
        VertexDeclaration vertexDeclaration;
        VertexBuffer vertexBuffer;

        public Game1()
        {
            graphics = new GraphicsDeviceManager(this);
            Content.RootDirectory = "Content";
        }

        /// <summary>
        /// Allows the game to perform any initialization it needs to before starting to run.
        /// This is where it can query for any required services and load any non-graphic
        /// related content.  Calling base.Initialize will enumerate through any components
        /// and initialize them as well.
        /// </summary>
        protected override void Initialize()
        {
            // TODO: Add your initialization logic here
            InitializeTransform();
            InitializeEffect();
            InitializeCubePrimitive();

            base.Initialize();
        }

        /// <summary>
        /// Initializes the transforms used by the game.
        /// </summary>
        private void InitializeTransform()
        {
            // world matrix
            world = Matrix.Identity;

            // view matrix
            view = Matrix.CreateLookAt(new Vector3(0, 0, 10.0f), Vector3.Zero, Vector3.Up);

            // projection matrix
            float aspectRatio = (float)GraphicsDevice.Viewport.Width / (float)GraphicsDevice.Viewport.Height;
            projection = Matrix.CreatePerspectiveFieldOfView(MathHelper.PiOver4, aspectRatio, 1.0f, 1000.0f);
        }

        /// <summary>
        /// Initializes the effect (loading, parameter setting, and technique selection)
        /// used by the game.
        /// </summary>
        private void InitializeEffect()
        {
            // Initialize Effect
            effect = new BasicEffect(GraphicsDevice, null);
            effect.VertexColorEnabled = true;
            effect.Alpha = 1.0f;

            effect.World = world;
            effect.View = view;
            effect.Projection = projection;
        }

        /// <summary>
        /// Initializes the cube geometry primitive.
        /// </summary>
        private void InitializeCubePrimitive()
        {
            vertexDeclaration = new VertexDeclaration(graphics.GraphicsDevice, VertexPositionColor.VertexElements);

            // vertex 생성
            vertices = new VertexPositionColor[8];
            vertices[0] = new VertexPositionColor(new Vector3(-1.0f, -1.0f, 1.0f), Color.White);
            vertices[1] = new VertexPositionColor(new Vector3(-1.0f, 1.0f, 1.0f), Color.Black);
            vertices[2] = new VertexPositionColor(new Vector3(1.0f, 1.0f, 1.0f), Color.Red);
            vertices[3] = new VertexPositionColor(new Vector3(1.0f, -1.0f, 1.0f), Color.Green);
            vertices[4] = new VertexPositionColor(new Vector3(-1.0f, -1.0f, -1.0f), Color.Blue);
            vertices[5] = new VertexPositionColor(new Vector3(-1.0f, 1.0f, -1.0f), Color.Yellow);
            vertices[6] = new VertexPositionColor(new Vector3(1.0f, 1.0f, -1.0f), Color.Cyan);
            vertices[7] = new VertexPositionColor(new Vector3(1.0f, -1.0f, -1.0f), Color.Magenta);

            // Index 생성
            line_Indices = new short[24] { 0, 1,  // 앞면
                                           1, 2, 
                                           2, 3,
                                           3, 0,
                                           4, 5,  // 뒷면
                                           5, 6,
                                           6, 7,
                                           7, 4,
                                           0, 4,  // 왼쪽나머지
                                           1, 5,
                                           2, 6,  // 오른쪽나머지
                                           3, 7
                                         };

            polygon_Indices = new short[36] {  0, 1, 2,   // 앞면
                                         0, 2, 3,
                                         4, 6, 5,   // 뒷면
                                         4, 7, 6,
                                         4, 5, 1,   // 왼쪽면
                                         4, 1, 0,
                                         3, 2, 6,   // 오른쪽면
                                         3, 6, 7, 
                                         1, 5, 6,   // 윗면
                                         1, 6, 2,
                                         4, 0, 3,   // 아랫면
                                         4, 3, 7
                                      };

            // Initialize the vertex buffer, allocating memory for each vertex.
            vertexBuffer = new VertexBuffer(graphics.GraphicsDevice,
                VertexPositionColor.SizeInBytes * (vertices.Length),
                BufferUsage.None);

            // Set the vertex buffer data to the array of vertices.
            vertexBuffer.SetData<VertexPositionColor>(vertices);
        }

        /// <summary>
        /// LoadContent will be called once per game and is the place to load
        /// all of your content.
        /// </summary>
        protected override void LoadContent()
        {
            // Create a new SpriteBatch, which can be used to draw textures.
            spriteBatch = new SpriteBatch(GraphicsDevice);

            // TODO: use this.Content to load your game content here
        }

        /// <summary>
        /// UnloadContent will be called once per game and is the place to unload
        /// all content.
        /// </summary>
        protected override void UnloadContent()
        {
            // TODO: Unload any non ContentManager content here
        }

        /// <summary>
        /// Allows the game to run logic such as updating the world,
        /// checking for collisions, gathering input, and playing audio.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Update(GameTime gameTime)
        {
            // Allows the game to exit
            if (GamePad.GetState(PlayerIndex.One).Buttons.Back == ButtonState.Pressed)
                this.Exit();

            HandleInput();

            base.Update(gameTime);
        }

        /// <summary>
        /// Determines which primitive to draw based on input from the keyboard
        /// or game pad.
        /// </summary>
        protected void HandleInput()
        {
            CheckGamePadInput();
            CheckKeyboardInput();
        }

        private void CheckGamePadInput()
        {
            prevGamePadState = currentGamePadState;
            currentGamePadState = GamePad.GetState(PlayerIndex.One);

            if (((currentGamePadState.Buttons.A == ButtonState.Pressed)) &&
                 (prevGamePadState.Buttons.A == ButtonState.Released))
            {
                fillMode = !fillMode;   // toggle fillmode
            }
        }

        private void CheckKeyboardInput()
        {
            prevKeyboardState = currentKeyboardState;
            currentKeyboardState = Keyboard.GetState(PlayerIndex.One);

            if (currentKeyboardState.IsKeyDown(Keys.Escape))
            {
                this.Exit();
            }
            else if (currentKeyboardState.IsKeyDown(Keys.W)
                && !prevKeyboardState.IsKeyDown(Keys.W))
            {
                fillMode = !fillMode;   // toggle wireframe/fillmode
            }
            else if (currentKeyboardState.IsKeyDown(Keys.X)
                && !prevKeyboardState.IsKeyDown(Keys.X))
            {
                angle += 5.0f;
            }
            else if (currentKeyboardState.IsKeyDown(Keys.Y)
                && !prevKeyboardState.IsKeyDown(Keys.Y))
            {
                zoom -= 0.01f;
            }
            else if (currentKeyboardState.IsKeyDown(Keys.Space)
                && !prevKeyboardState.IsKeyDown(Keys.Space))
            {
                animationMode = !animationMode;
            }
        }

        /// <summary>
        /// This is called when the game should draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Draw(GameTime gameTime)
        {
            GraphicsDevice.Clear(Color.CornflowerBlue);

            // TODO: Add your drawing code here
            graphics.GraphicsDevice.VertexDeclaration = vertexDeclaration;
            if (animationMode) angle += 0.8f;

            // draw cube1
            // world matrix
            world = Matrix.CreateRotationY(MathHelper.ToRadians(angle)) * Matrix.CreateTranslation(0, 0, zoom) * Matrix.CreateTranslation(-2, 0, 0);
            effect.World = world;
            // effect
            effect.Begin(SaveStateMode.SaveState);
            foreach (EffectPass pass in effect.CurrentTechnique.Passes)
            {
                pass.Begin();
                if (fillMode)
                    graphics.GraphicsDevice.DrawUserIndexedPrimitives<VertexPositionColor>(PrimitiveType.TriangleList, vertices, 0, 8, polygon_Indices, 0, 12);
                else
                    graphics.GraphicsDevice.DrawUserIndexedPrimitives<VertexPositionColor>(PrimitiveType.LineList, vertices, 0, 8, line_Indices, 0, 12);
                pass.End();
            }
            effect.End();

            // draw cube2
            // world matrix
            world = Matrix.CreateRotationY(MathHelper.ToRadians(angle)) * Matrix.CreateTranslation(0, 0, zoom) * Matrix.CreateTranslation(2, 0, 0);
            effect.World = world;
            // effect
            effect.Begin(SaveStateMode.SaveState);
            foreach (EffectPass pass in effect.CurrentTechnique.Passes)
            {
                pass.Begin();
                if (fillMode)
                    graphics.GraphicsDevice.DrawUserIndexedPrimitives<VertexPositionColor>(PrimitiveType.TriangleList, vertices, 0, 8, polygon_Indices, 0, 12);
                else
                    graphics.GraphicsDevice.DrawUserIndexedPrimitives<VertexPositionColor>(PrimitiveType.LineList, vertices, 0, 8, line_Indices, 0, 12);
                pass.End();
            }
            effect.End();

            base.Draw(gameTime);
        }
    }
}
